@@ -9,7 +9,7 @@
 #include <stdio.h>
 #include <assert.h>
 #include <string.h>
-//#define DEBUG
+// #define DEBUG
 #include "debug.h"
 #define UNASSIGNED 3U
 #define NULL_EDGE 0xffffffff
@@ -35,9 +35,9 @@ const cmph_uint8 bdz_lookup_table[] =
 3, 3, 3, 2, 3, 3, 3, 2, 3, 3, 3, 2, 2, 2, 2, 1,
 3, 3, 3, 2, 3, 3, 3, 2, 3, 3, 3, 2, 2, 2, 2, 1,
 2, 2, 2, 1, 2, 2, 2, 1, 2, 2, 2, 1, 1, 1, 1, 0
-};	
+};
 
-typedef struct 
+typedef struct
 {
 	cmph_uint32 vertices[3];
 	cmph_uint32 next_edges[3];
@@ -47,27 +47,27 @@ typedef cmph_uint32 * bdz_queue_t;
 
 static void bdz_alloc_queue(bdz_queue_t * queuep, cmph_uint32 nedges)
 {
-	(*queuep)=malloc(nedges*sizeof(cmph_uint32));
+	(*queuep)=(cmph_uint32 *)malloc(nedges*sizeof(cmph_uint32));
 };
 static void bdz_free_queue(bdz_queue_t * queue)
 {
 	free(*queue);
 };
 
-typedef struct 
+typedef struct
 {
 	cmph_uint32 nedges;
 	bdz_edge_t * edges;
 	cmph_uint32 * first_edge;
-	cmph_uint8 * vert_degree;	
+	cmph_uint8 * vert_degree;
 }bdz_graph3_t;
 
 
 static void bdz_alloc_graph3(bdz_graph3_t * graph3, cmph_uint32 nedges, cmph_uint32 nvertices)
 {
-	graph3->edges=malloc(nedges*sizeof(bdz_edge_t));
-	graph3->first_edge=malloc(nvertices*sizeof(cmph_uint32));
-	graph3->vert_degree=malloc((size_t)nvertices);	
+	graph3->edges=(bdz_edge_t *)malloc(nedges*sizeof(bdz_edge_t));
+	graph3->first_edge=(cmph_uint32 *)malloc(nvertices*sizeof(cmph_uint32));
+	graph3->vert_degree=(cmph_uint8 *)malloc((size_t)nvertices);
 };
 static void bdz_init_graph3(bdz_graph3_t * graph3, cmph_uint32 nedges, cmph_uint32 nvertices)
 {
@@ -115,10 +115,12 @@ static void bdz_dump_graph(bdz_graph3_t* graph3, cmph_uint32 nedges, cmph_uint32
 				graph3->edges[i].next_edges[1],graph3->edges[i].next_edges[2]);
 	};
 	
+        #ifdef DEBUG
 	for(i=0;i<nvertices;i++){
 		printf("\nfirst for vertice %d %d ",i,graph3->first_edge[i]);
 	
 	};
+        #endif
 };
 
 static void bdz_remove_edge(bdz_graph3_t * graph3, cmph_uint32 curr_edge)
@@ -134,7 +136,7 @@ static void bdz_remove_edge(bdz_graph3_t * graph3, cmph_uint32 curr_edge)
 				j=0;
 			} else if(graph3->edges[edge1].vertices[1]==vert){
 				j=1;
-			} else 
+			} else
 				j=2;
 			edge1=graph3->edges[edge1].next_edges[j];
 		};
@@ -143,16 +145,16 @@ static void bdz_remove_edge(bdz_graph3_t * graph3, cmph_uint32 curr_edge)
 			bdz_dump_graph(graph3,graph3->nedges,graph3->nedges+graph3->nedges/4);
 			exit(-1);
 		};
-		
+
 		if(edge2!=NULL_EDGE){
-			graph3->edges[edge2].next_edges[j] = 
+			graph3->edges[edge2].next_edges[j] =
 				graph3->edges[edge1].next_edges[i];
-		} else 
+		} else
 			graph3->first_edge[vert]=
 				graph3->edges[edge1].next_edges[i];
 		graph3->vert_degree[vert]--;
 	};
-	
+
 };
 
 static int bdz_generate_queue(cmph_uint32 nedges, cmph_uint32 nvertices, bdz_queue_t queue, bdz_graph3_t* graph3)
@@ -161,14 +163,14 @@ static int bdz_generate_queue(cmph_uint32 nedges, cmph_uint32 nvertices, bdz_que
 	cmph_uint32 queue_head=0,queue_tail=0;
 	cmph_uint32 curr_edge;
 	cmph_uint32 tmp_edge;
-	cmph_uint8 * marked_edge =malloc((size_t)(nedges >> 3) + 1);
+	cmph_uint8 * marked_edge = (cmph_uint8 *)malloc((size_t)(nedges >> 3) + 1);
 	memset(marked_edge, 0, (size_t)(nedges >> 3) + 1);
 
 	for(i=0;i<nedges;i++){
 		v0=graph3->edges[i].vertices[0];
 		v1=graph3->edges[i].vertices[1];
 		v2=graph3->edges[i].vertices[2];
-		if(graph3->vert_degree[v0]==1 || 
+		if(graph3->vert_degree[v0]==1 ||
 				graph3->vert_degree[v1]==1 ||
 				graph3->vert_degree[v2]==1){
 			if(!GETBIT(marked_edge,i)) {
@@ -177,9 +179,14 @@ static int bdz_generate_queue(cmph_uint32 nedges, cmph_uint32 nvertices, bdz_que
 			}
 		};
 	};
+        DEBUGP("Queue head %d Queue tail %d\n", queue_head, queue_tail);
+        #ifdef DEBUG
+	bdz_dump_graph(graph3,graph3->nedges,graph3->nedges+graph3->nedges/4);
+        #endif
 	while(queue_tail!=queue_head){
 		curr_edge=queue[queue_tail++];
 		bdz_remove_edge(graph3,curr_edge);
+		DEBUGP("Removing edge %d\n", curr_edge);
 		v0=graph3->edges[curr_edge].vertices[0];
 		v1=graph3->edges[curr_edge].vertices[1];
 		v2=graph3->edges[curr_edge].vertices[2];
@@ -189,7 +196,7 @@ static int bdz_generate_queue(cmph_uint32 nedges, cmph_uint32 nvertices, bdz_que
 				queue[queue_head++]=tmp_edge;
 				SETBIT(marked_edge,tmp_edge);
 			};
-			
+
 		};
 		if(graph3->vert_degree[v1]==1) {
 			tmp_edge=graph3->first_edge[v1];
@@ -197,7 +204,7 @@ static int bdz_generate_queue(cmph_uint32 nedges, cmph_uint32 nvertices, bdz_que
 				queue[queue_head++]=tmp_edge;
 				SETBIT(marked_edge,tmp_edge);
 			};
-			
+
 		};
 		if(graph3->vert_degree[v2]==1){
 			tmp_edge=graph3->first_edge[v2];
@@ -220,7 +227,7 @@ bdz_config_data_t *bdz_config_new(void)
 {
 	bdz_config_data_t *bdz;
 	bdz = (bdz_config_data_t *)malloc(sizeof(bdz_config_data_t));
-	assert(bdz);
+        if (!bdz) return NULL;
 	memset(bdz, 0, sizeof(bdz_config_data_t));
 	bdz->hashfunc = CMPH_HASH_JENKINS;
 	bdz->g = NULL;
@@ -321,10 +328,10 @@ cmph_t *bdz_new(cmph_config_t *mph, double c)
 				fprintf(stderr, "acyclic graph creation failure - %u iterations remaining\n", iterations);
 			}
 			if (iterations == 0) break;
-		} 
+		}
 		else break;
 	}
-	
+
 	if (iterations == 0)
 	{
 		bdz_free_queue(&edges);
@@ -346,7 +353,7 @@ cmph_t *bdz_new(cmph_config_t *mph, double c)
 		fprintf(stderr, "Entering ranking step for mph creation of %u keys with graph sized %u\n", bdz->m, bdz->n);
 	}
 	ranking(bdz);
-	#ifdef CMPH_TIMING	
+	#ifdef CMPH_TIMING
 	ELAPSED_TIME_IN_SECONDS(&construction_time);
 	#endif
 	mphf = (cmph_t *)malloc(sizeof(cmph_t));
@@ -374,17 +381,17 @@ cmph_t *bdz_new(cmph_config_t *mph, double c)
 	}
 
 
-	#ifdef CMPH_TIMING	
+	#ifdef CMPH_TIMING
 	register cmph_uint32 space_usage = bdz_packed_size(mphf)*8;
 	register cmph_uint32 keys_per_bucket = 1;
 	construction_time = construction_time - construction_time_begin;
 	fprintf(stdout, "%u\t%.2f\t%u\t%.4f\t%.4f\n", bdz->m, bdz->m/(double)bdz->n, keys_per_bucket, construction_time, space_usage/(double)bdz->m);
-	#endif	
+	#endif
 
 	return mphf;
 }
 
-		
+
 static int bdz_mapping(cmph_config_t *mph, bdz_graph3_t* graph3, bdz_queue_t queue)
 {
 	cmph_uint32 e;
@@ -398,15 +405,16 @@ static int bdz_mapping(cmph_config_t *mph, bdz_graph3_t* graph3, bdz_queue_t que
 		cmph_uint32 h0, h1, h2;
 		cmph_uint32 keylen;
 		char *key = NULL;
-		mph->key_source->read(mph->key_source->data, &key, &keylen);		
+		mph->key_source->read(mph->key_source->data, &key, &keylen);
 		hash_vector(bdz->hl, key, keylen,hl);
 		h0 = hl[0] % bdz->r;
 		h1 = hl[1] % bdz->r + bdz->r;
 		h2 = hl[2] % bdz->r + (bdz->r << 1);
+                DEBUGP("Key: %.*s (%u %u %u)\n", keylen, key, h0, h1, h2);
 		mph->key_source->dispose(mph->key_source->data, key, keylen);
 		bdz_add_edge(graph3,h0,h1,h2);
 	}
-	cycles = bdz_generate_queue(bdz->m, bdz->n, queue, graph3);	
+	cycles = bdz_generate_queue(bdz->m, bdz->n, queue, graph3);
 	return (cycles == 0);
 }
 
@@ -416,9 +424,9 @@ static void assigning(bdz_config_data_t *bdz, bdz_graph3_t* graph3, bdz_queue_t
 	cmph_uint32 nedges=graph3->nedges;
 	cmph_uint32 curr_edge;
 	cmph_uint32 v0,v1,v2;
-	cmph_uint8 * marked_vertices =malloc((size_t)(bdz->n >> 3) + 1);
+	cmph_uint8 * marked_vertices = (cmph_uint8 *)malloc((size_t)(bdz->n >> 3) + 1);
         cmph_uint32 sizeg = (cmph_uint32)ceil(bdz->n/4.0);
-	bdz->g = (cmph_uint8 *)calloc((size_t)(sizeg), sizeof(cmph_uint8));	
+	bdz->g = (cmph_uint8 *)calloc((size_t)(sizeg), sizeof(cmph_uint8));
 	memset(marked_vertices, 0, (size_t)(bdz->n >> 3) + 1);
 	memset(bdz->g, 0xff, (size_t)(sizeg));
 
@@ -427,16 +435,16 @@ static void assigning(bdz_config_data_t *bdz, bdz_graph3_t* graph3, bdz_queue_t
 		v0=graph3->edges[curr_edge].vertices[0];
 		v1=graph3->edges[curr_edge].vertices[1];
 		v2=graph3->edges[curr_edge].vertices[2];
-		DEBUGP("B:%u %u %u -- %u %u %u\n", v0, v1, v2, GETVALUE(bdz->g, v0), GETVALUE(bdz->g, v1), GETVALUE(bdz->g, v2));
+		DEBUGP("B:%u %u %u -- %u %u %u edge %u\n", v0, v1, v2, GETVALUE(bdz->g, v0), GETVALUE(bdz->g, v1), GETVALUE(bdz->g, v2), curr_edge);
 		if(!GETBIT(marked_vertices, v0)){
 			if(!GETBIT(marked_vertices,v1))
 			{
-				SETVALUE1(bdz->g, v1, UNASSIGNED); 
+				SETVALUE1(bdz->g, v1, UNASSIGNED);
 				SETBIT(marked_vertices, v1);
 			}
 			if(!GETBIT(marked_vertices,v2))
 			{
-				SETVALUE1(bdz->g, v2, UNASSIGNED);		
+				SETVALUE1(bdz->g, v2, UNASSIGNED);
 				SETBIT(marked_vertices, v2);
 			}
 			SETVALUE1(bdz->g, v0, (6-(GETVALUE(bdz->g, v1) + GETVALUE(bdz->g,v2)))%3);
@@ -499,7 +507,7 @@ int bdz_dump(cmph_t *mphf, FILE *fd)
 	nbytes = fwrite(&(data->n), sizeof(cmph_uint32), (size_t)1, fd);
 	nbytes = fwrite(&(data->m), sizeof(cmph_uint32), (size_t)1, fd);
 	nbytes = fwrite(&(data->r), sizeof(cmph_uint32), (size_t)1, fd);
-	
+
 	cmph_uint32 sizeg = (cmph_uint32)ceil(data->n/4.0);
 	nbytes = fwrite(data->g, sizeof(cmph_uint8)*sizeg, (size_t)1, fd);
 
@@ -533,12 +541,12 @@ void bdz_load(FILE *f, cmph_t *mphf)
 	nbytes = fread(buf, (size_t)buflen, (size_t)1, f);
 	bdz->hl = hash_state_load(buf, buflen);
 	free(buf);
-	
+
 
 	DEBUGP("Reading m and n\n");
-	nbytes = fread(&(bdz->n), sizeof(cmph_uint32), (size_t)1, f);	
-	nbytes = fread(&(bdz->m), sizeof(cmph_uint32), (size_t)1, f);	
-	nbytes = fread(&(bdz->r), sizeof(cmph_uint32), (size_t)1, f);	
+	nbytes = fread(&(bdz->n), sizeof(cmph_uint32), (size_t)1, f);
+	nbytes = fread(&(bdz->m), sizeof(cmph_uint32), (size_t)1, f);
+	nbytes = fread(&(bdz->r), sizeof(cmph_uint32), (size_t)1, f);
 	sizeg = (cmph_uint32)ceil(bdz->n/4.0);
 	bdz->g = (cmph_uint8 *)calloc((size_t)(sizeg), sizeof(cmph_uint8));
 	nbytes = fread(bdz->g, sizeg*sizeof(cmph_uint8), (size_t)1, f);
@@ -558,7 +566,7 @@ void bdz_load(FILE *f, cmph_t *mphf)
 	#endif
 	return;
 }
-		
+
 
 static inline cmph_uint32 rank(cmph_uint32 b, cmph_uint32 * ranktable, cmph_uint8 * g, cmph_uint32 vertex)
 {
@@ -570,28 +578,31 @@ static inline cmph_uint32 rank(cmph_uint32 b, cmph_uint32 * ranktable, cmph_uint
 	while(beg_idx_b < end_idx_b)
 	{
 		base_rank += bdz_lookup_table[*(g + beg_idx_b++)];
-		
+
 	}
+        DEBUGP("base rank %u\n", base_rank);
 	beg_idx_v = beg_idx_b << 2;
-	while(beg_idx_v < vertex) 
+        DEBUGP("beg_idx_v %u\n", beg_idx_v);
+	while(beg_idx_v < vertex)
 	{
 		if(GETVALUE(g, beg_idx_v) != UNASSIGNED) base_rank++;
 		beg_idx_v++;
 	}
-	
+
 	return base_rank;
 }
 
 cmph_uint32 bdz_search(cmph_t *mphf, const char *key, cmph_uint32 keylen)
 {
 	register cmph_uint32 vertex;
-	register bdz_data_t *bdz = mphf->data;
+	register bdz_data_t *bdz = (bdz_data_t *)mphf->data;
 	cmph_uint32 hl[3];
 	hash_vector(bdz->hl, key, keylen, hl);
 	hl[0] = hl[0] % bdz->r;
 	hl[1] = hl[1] % bdz->r + bdz->r;
 	hl[2] = hl[2] % bdz->r + (bdz->r << 1);
 	vertex = hl[(GETVALUE(bdz->g, hl[0]) + GETVALUE(bdz->g, hl[1]) + GETVALUE(bdz->g, hl[2])) % 3];
+        DEBUGP("Search found vertex %u\n", vertex);
 	return rank(bdz->b, bdz->ranktable, bdz->g, vertex);
 }
 
@@ -599,7 +610,7 @@ cmph_uint32 bdz_search(cmph_t *mphf, const char *key, cmph_uint32 keylen)
 void bdz_destroy(cmph_t *mphf)
 {
 	bdz_data_t *data = (bdz_data_t *)mphf->data;
-	free(data->g);	
+	free(data->g);
 	hash_state_destroy(data->hl);
 	free(data->ranktable);
 	free(data);
@@ -614,7 +625,7 @@ void bdz_destroy(cmph_t *mphf)
 void bdz_pack(cmph_t *mphf, void *packed_mphf)
 {
 	bdz_data_t *data = (bdz_data_t *)mphf->data;
-	cmph_uint8 * ptr = packed_mphf;
+	cmph_uint8 * ptr = (cmph_uint8 *)packed_mphf;
 
 	// packing hl type
 	CMPH_HASH hl_type = hash_get_type(data->hl);
@@ -649,18 +660,18 @@ void bdz_pack(cmph_t *mphf, void *packed_mphf)
  *  \brief Return the amount of space needed to pack mphf.
  *  \param mphf pointer to a mphf
  *  \return the size of the packed function or zero for failures
- */ 
+ */
 cmph_uint32 bdz_packed_size(cmph_t *mphf)
 {
 	bdz_data_t *data = (bdz_data_t *)mphf->data;
 
-	CMPH_HASH hl_type = hash_get_type(data->hl); 
+	CMPH_HASH hl_type = hash_get_type(data->hl);
 
 	return (cmph_uint32)(sizeof(CMPH_ALGO) + hash_state_packed_size(hl_type) + 3*sizeof(cmph_uint32) + sizeof(cmph_uint32)*(data->ranktablesize) + sizeof(cmph_uint8) + sizeof(cmph_uint8)* (cmph_uint32)(ceil(data->n/4.0)));
 }
 
 /** cmph_uint32 bdz_search(void *packed_mphf, const char *key, cmph_uint32 keylen);
- *  \brief Use the packed mphf to do a search. 
+ *  \brief Use the packed mphf to do a search.
  *  \param  packed_mphf pointer to the packed mphf
  *  \param key key to be hashed
  *  \param keylen key legth in bytes
@@ -668,13 +679,13 @@ cmph_uint32 bdz_packed_size(cmph_t *mphf)
  */
 cmph_uint32 bdz_search_packed(void *packed_mphf, const char *key, cmph_uint32 keylen)
 {
-	
+
 	register cmph_uint32 vertex;
-	register CMPH_HASH hl_type  = *(cmph_uint32 *)packed_mphf;
+	register CMPH_HASH hl_type  = (CMPH_HASH)(*(cmph_uint32 *)packed_mphf);
 	register cmph_uint8 *hl_ptr = (cmph_uint8 *)(packed_mphf) + 4;
 
 	register cmph_uint32 *ranktable = (cmph_uint32*)(hl_ptr + hash_state_packed_size(hl_type));
-	
+
 	register cmph_uint32 r = *ranktable++;
 	register cmph_uint32 ranktablesize = *ranktable++;
 	register cmph_uint8 * g = (cmph_uint8 *)(ranktable + ranktablesize);
