@@ -29,7 +29,7 @@ struct _chd_ph_item_t
 };
 typedef struct _chd_ph_item_t chd_ph_item_t;
 
-// struct to represent the items at mapping phase only. 
+// struct to represent the items at mapping phase only.
 struct _chd_ph_map_item_t
 {
 	cmph_uint32 f;
@@ -77,7 +77,7 @@ void chd_ph_bucket_clean(chd_ph_bucket_t * buckets, cmph_uint32 nbuckets)
 	for(i = 0; i < nbuckets; i++)
 		buckets[i].size = 0;
 }
-cmph_uint8 chd_ph_bucket_insert(chd_ph_bucket_t * buckets,chd_ph_map_item_t * map_items, chd_ph_item_t * items,
+static cmph_uint8 chd_ph_bucket_insert(chd_ph_bucket_t * buckets,chd_ph_map_item_t * map_items, chd_ph_item_t * items,
 				cmph_uint32 nbuckets,cmph_uint32 item_idx)
 {
 	register cmph_uint32 i = 0;
@@ -85,7 +85,7 @@ cmph_uint8 chd_ph_bucket_insert(chd_ph_bucket_t * buckets,chd_ph_map_item_t * ma
 	register chd_ph_map_item_t * tmp_map_item = map_items + item_idx;
 	register chd_ph_bucket_t * bucket = buckets + tmp_map_item->bucket_num;
 	tmp_item = items + bucket->items_list;
-	
+
 	for(i = 0; i < bucket->size; i++)
 	{
 		if(tmp_item->f == tmp_map_item->f && tmp_item->h == tmp_map_item->h)
@@ -105,7 +105,7 @@ void chd_ph_bucket_destroy(chd_ph_bucket_t * buckets)
     free(buckets);
 }
 
-static inline cmph_uint8 chd_ph_mapping(cmph_config_t *mph, chd_ph_bucket_t * buckets, chd_ph_item_t * items, 
+static inline cmph_uint8 chd_ph_mapping(cmph_config_t *mph, chd_ph_bucket_t * buckets, chd_ph_item_t * items,
 					cmph_uint32 *max_bucket_size);
 
 static chd_ph_sorted_list_t * chd_ph_ordering(chd_ph_bucket_t ** _buckets,chd_ph_item_t ** items,
@@ -131,7 +131,7 @@ static inline double chd_ph_get_entropy(cmph_uint32 * disp_table, cmph_uint32 n,
 	{
 		probe_counts[disp_table[i]]++;
 	};
-	
+
 	for(i = 0; i < max_probes; i++)
 	{
 		if(probe_counts[i] > 0)
@@ -141,13 +141,13 @@ static inline double chd_ph_get_entropy(cmph_uint32 * disp_table, cmph_uint32 n,
 	return entropy;
 };
 
-chd_ph_config_data_t *chd_ph_config_new()
+chd_ph_config_data_t *chd_ph_config_new(void)
 {
 	chd_ph_config_data_t *chd_ph;
 	chd_ph = (chd_ph_config_data_t *)malloc(sizeof(chd_ph_config_data_t));
-	assert(chd_ph);
+        if (!chd_ph) return NULL;
 	memset(chd_ph, 0, sizeof(chd_ph_config_data_t));
-	
+
 	chd_ph->hashfunc = CMPH_HASH_JENKINS;
 	chd_ph->cs = NULL;
 	chd_ph->nbuckets = 0;
@@ -159,7 +159,7 @@ chd_ph_config_data_t *chd_ph_config_new()
 	chd_ph->keys_per_bin = 1;
 	chd_ph->keys_per_bucket = 4;
 	chd_ph->occup_table = 0;
-	
+
 	return chd_ph;
 }
 
@@ -184,7 +184,7 @@ void chd_ph_config_set_hashfuncs(cmph_config_t *mph, CMPH_HASH *hashfuncs)
 	while(*hashptr != CMPH_HASH_COUNT)
 	{
 		if (i >= 1) break; //chd_ph only uses one linear hash function
-		chd_ph->hashfunc = *hashptr;	
+		chd_ph->hashfunc = *hashptr;
 		++i, ++hashptr;
 	}
 }
@@ -221,31 +221,31 @@ cmph_uint8 chd_ph_mapping(cmph_config_t *mph, chd_ph_bucket_t * buckets, chd_ph_
 	char * key = NULL;
 	cmph_uint32 keylen = 0;
 	chd_ph_map_item_t * map_item;
-	chd_ph_map_item_t * map_items = malloc(chd_ph->m*sizeof(chd_ph_map_item_t));
+	chd_ph_map_item_t * map_items = (chd_ph_map_item_t *)malloc(chd_ph->m*sizeof(chd_ph_map_item_t));
 	register cmph_uint32 mapping_iterations = 1000;
 	*max_bucket_size = 0;
 	while(1)
 	{
 		mapping_iterations--;
 		if (chd_ph->hl) hash_state_destroy(chd_ph->hl);
-		chd_ph->hl = hash_state_new(chd_ph->hashfunc, chd_ph->m); 
+		chd_ph->hl = hash_state_new(chd_ph->hashfunc, chd_ph->m);
 
 		chd_ph_bucket_clean(buckets, chd_ph->nbuckets);
 
-		mph->key_source->rewind(mph->key_source->data);  
+		mph->key_source->rewind(mph->key_source->data);
 
 		for(i = 0; i < chd_ph->m; i++)
 		{
-			mph->key_source->read(mph->key_source->data, &key, &keylen);		
+			mph->key_source->read(mph->key_source->data, &key, &keylen);
 			hash_vector(chd_ph->hl, key, keylen, hl);
-			
+
 			map_item = (map_items + i);
 
 			g = hl[0] % chd_ph->nbuckets;
 			map_item->f = hl[1] % chd_ph->n;
 			map_item->h = hl[2] % (chd_ph->n - 1) + 1;
 			map_item->bucket_num=g;
-			mph->key_source->dispose(mph->key_source->data, key, keylen);		
+			mph->key_source->dispose(mph->key_source->data, key, keylen);
 // 			if(buckets[g].size == (chd_ph->keys_per_bucket << 2))
 // 			{
 // 				DEBUGP("BUCKET = %u -- SIZE = %u -- MAXIMUM SIZE = %u\n", g, buckets[g].size, (chd_ph->keys_per_bucket << 2));
@@ -275,7 +275,7 @@ cmph_uint8 chd_ph_mapping(cmph_config_t *mph, chd_ph_bucket_t * buckets, chd_ph_
 			free(map_items);
 			return 1; // SUCCESS
 		}
-		
+
 		if(mapping_iterations == 0)
 		{
 		      goto error;
@@ -292,7 +292,7 @@ chd_ph_sorted_list_t * chd_ph_ordering(chd_ph_bucket_t ** _buckets, chd_ph_item_
 	cmph_uint32 nbuckets, cmph_uint32 nitems, cmph_uint32 max_bucket_size)
 {
 	chd_ph_sorted_list_t * sorted_lists = (chd_ph_sorted_list_t *) calloc(max_bucket_size + 1, sizeof(chd_ph_sorted_list_t));
-	
+
 	chd_ph_bucket_t * input_buckets = (*_buckets);
 	chd_ph_bucket_t * output_buckets;
 	chd_ph_item_t * input_items = (*_items);
@@ -317,9 +317,9 @@ chd_ph_sorted_list_t * chd_ph_ordering(chd_ph_bucket_t ** _buckets, chd_ph_item_
 	};
 	sorted_lists[i-1].size = 0;
 	// Store the buckets in a new array which is sorted by bucket sizes
-	output_buckets = calloc(nbuckets, sizeof(chd_ph_bucket_t)); // everything is initialized with zero
+	output_buckets = (chd_ph_bucket_t *)calloc(nbuckets, sizeof(chd_ph_bucket_t)); // everything is initialized with zero
 //  	non_empty_buckets = nbuckets;
-	
+
 	for(i = 0; i < nbuckets; i++)
 	{
 		bucket_size = input_buckets[i].size;
@@ -338,8 +338,8 @@ chd_ph_sorted_list_t * chd_ph_ordering(chd_ph_bucket_t ** _buckets, chd_ph_item_
 	// Return the buckets sorted in new order and free the old buckets sorted in old order
 	free(input_buckets);
 	(*_buckets) = output_buckets;
-	
-	
+
+
 	// Store the items according to the new order of buckets.
 	output_items = (chd_ph_item_t*)calloc(nitems, sizeof(chd_ph_item_t));
 	position = 0;
@@ -426,26 +426,26 @@ static inline cmph_uint8 place_bucket_probe(chd_ph_config_data_t *chd_ph, chd_ph
 				}
 				position = (cmph_uint32)((item->f + ((cmph_uint64 )item->h) * probe0_num + probe1_num) % chd_ph->n);
 				UNSETBIT32(((cmph_uint32*)chd_ph->occup_table), position);
-				
+
 // 				([position/32]^=(1<<(position%32));
 				item++;
 				i--;
 			};
 		};
 		return 0;
-	} 	
+	} 
 	return 1;
 };
 
-static inline cmph_uint8 place_bucket(chd_ph_config_data_t *chd_ph, chd_ph_bucket_t *buckets, chd_ph_item_t * items, cmph_uint32 max_probes, 
+static inline cmph_uint8 place_bucket(chd_ph_config_data_t *chd_ph, chd_ph_bucket_t *buckets, chd_ph_item_t * items, cmph_uint32 max_probes,
                                       cmph_uint32 * disp_table, cmph_uint32 bucket_num, cmph_uint32 size)
-				      
+
 {
 	register cmph_uint32 probe0_num, probe1_num, probe_num;
 	probe0_num = 0;
 	probe1_num = 0;
 	probe_num = 0;
-	
+
 	while(1)
 	{
 		if(place_bucket_probe(chd_ph, buckets, items, probe0_num, probe1_num, bucket_num,size))
@@ -469,7 +469,7 @@ static inline cmph_uint8 place_bucket(chd_ph_config_data_t *chd_ph, chd_ph_bucke
 };
 
 static inline cmph_uint8 place_buckets1(chd_ph_config_data_t *chd_ph, chd_ph_bucket_t * buckets, chd_ph_item_t *items,
-					cmph_uint32 max_bucket_size, chd_ph_sorted_list_t *sorted_lists, cmph_uint32 max_probes, 
+					cmph_uint32 max_bucket_size, chd_ph_sorted_list_t *sorted_lists, cmph_uint32 max_probes,
 					cmph_uint32 * disp_table)
 {
 	register cmph_uint32 i = 0;
@@ -490,8 +490,8 @@ static inline cmph_uint8 place_buckets1(chd_ph_config_data_t *chd_ph, chd_ph_buc
 	return 1;
 };
 
-static inline cmph_uint8 place_buckets2(chd_ph_config_data_t *chd_ph, chd_ph_bucket_t *buckets, chd_ph_item_t * items, 
-					cmph_uint32 max_bucket_size, chd_ph_sorted_list_t *sorted_lists, cmph_uint32 max_probes, 
+static inline cmph_uint8 place_buckets2(chd_ph_config_data_t *chd_ph, chd_ph_bucket_t *buckets, chd_ph_item_t * items,
+					cmph_uint32 max_bucket_size, chd_ph_sorted_list_t *sorted_lists, cmph_uint32 max_probes,
 					cmph_uint32 * disp_table)
 {
 	register cmph_uint32 i,j, non_placed_bucket;
@@ -516,10 +516,10 @@ static inline cmph_uint8 place_buckets2(chd_ph_config_data_t *chd_ph, chd_ph_buc
 			{
 				// if bucket is successfully placed remove it from list
 				if(place_bucket_probe(chd_ph, buckets, items, probe0_num, probe1_num, curr_bucket, i))
-				{	
+				{
 					disp_table[buckets[curr_bucket].bucket_id] = probe0_num + probe1_num * chd_ph->n;
 // 					DEBUGP("BUCKET %u PLACED --- DISPLACEMENT = %u\n", curr_bucket, disp_table[curr_bucket]);
-				} 
+				}
 				else
 				{
 // 					DEBUGP("BUCKET %u NOT PLACED\n", curr_bucket);
@@ -529,7 +529,7 @@ static inline cmph_uint8 place_buckets2(chd_ph_config_data_t *chd_ph, chd_ph_buc
 #endif
 					buckets[non_placed_bucket + sorted_lists[i].buckets_list].items_list = buckets[curr_bucket].items_list;
 					buckets[non_placed_bucket + sorted_lists[i].buckets_list].bucket_id = buckets[curr_bucket].bucket_id;
-#ifdef DEBUG		
+#ifdef DEBUG
 					buckets[curr_bucket].items_list=items_list;
 					buckets[curr_bucket].bucket_id=bucket_id;
 #endif
@@ -557,7 +557,7 @@ static inline cmph_uint8 place_buckets2(chd_ph_config_data_t *chd_ph, chd_ph_buc
 };
 
 cmph_uint8 chd_ph_searching(chd_ph_config_data_t *chd_ph, chd_ph_bucket_t *buckets, chd_ph_item_t *items ,
-			    cmph_uint32 max_bucket_size, chd_ph_sorted_list_t *sorted_lists, cmph_uint32 max_probes, 
+			    cmph_uint32 max_bucket_size, chd_ph_sorted_list_t *sorted_lists, cmph_uint32 max_probes,
 			    cmph_uint32 * disp_table)
 {
 	if(chd_ph->use_h)
@@ -582,7 +582,7 @@ static inline cmph_uint8 chd_ph_check_bin_hashing(chd_ph_config_data_t *chd_ph,
 		memset(chd_ph->occup_table, 0, chd_ph->n);
 	else
 		memset(chd_ph->occup_table, 0, ((chd_ph->n + 31)/32) * sizeof(cmph_uint32));
-		
+
 	for(bucket_size = 1; bucket_size <= max_bucket_size; bucket_size++)
 		for(i = sorted_lists[bucket_size].buckets_list; i < sorted_lists[bucket_size].size +
 				sorted_lists[bucket_size].buckets_list; i++)
@@ -602,7 +602,7 @@ static inline cmph_uint8 chd_ph_check_bin_hashing(chd_ph_config_data_t *chd_ph,
 						return 0;
 					}
 					(chd_ph->occup_table[position])++;
-				} 
+				}
 				else
 				{
 					if(GETBIT32(((cmph_uint32*)chd_ph->occup_table), position))
@@ -624,7 +624,7 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 	cmph_t *mphf = NULL;
 	chd_ph_data_t *chd_phf = NULL;
 	chd_ph_config_data_t *chd_ph = (chd_ph_config_data_t *)mph->data;
-	
+
 	register double load_factor = c;
 	register cmph_uint8 searching_success = 0;
 	register cmph_uint32 max_probes = 1 << 20; // default value for max_probes
@@ -645,24 +645,24 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 
 	chd_ph->m = mph->key_source->nkeys;
 	DEBUGP("m = %u\n", chd_ph->m);
-	
+
 	chd_ph->nbuckets = (cmph_uint32)(chd_ph->m/chd_ph->keys_per_bucket) + 1;
 	DEBUGP("nbuckets = %u\n", chd_ph->nbuckets);
-	
+
 	if(load_factor < 0.5 )
 	{
 		load_factor = 0.5;
 	}
-	
+
 	if(load_factor >= 0.99)
 	{
 		load_factor = 0.99;
 	}
-	
+
 	DEBUGP("load_factor = %.3f\n", load_factor);
-	
+
 	chd_ph->n = (cmph_uint32)(chd_ph->m/(chd_ph->keys_per_bin * load_factor)) + 1;
-	
+
 	//Round the number of bins to the prime immediately above
 	if(chd_ph->n % 2 == 0) chd_ph->n++;
 	for(;;)
@@ -670,35 +670,35 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 		if(check_primality(chd_ph->n) == 1)
 			break;
 		chd_ph->n += 2; // just odd numbers can be primes for n > 2
-		
+
 	};
-	
+
 	DEBUGP("n = %u \n", chd_ph->n);
 	if(chd_ph->keys_per_bin == 1)
 	{
 		space_lower_bound = chd_ph_space_lower_bound(chd_ph->m, chd_ph->n);
 	}
-	
+
 	if(mph->verbosity)
 	{
 		fprintf(stderr, "space lower bound is %.3f bits per key\n", space_lower_bound);
 	}
 
        	// We allocate the working tables
-	buckets = chd_ph_bucket_new(chd_ph->nbuckets); 
+	buckets = chd_ph_bucket_new(chd_ph->nbuckets);
 	items   = (chd_ph_item_t *) calloc(chd_ph->m, sizeof(chd_ph_item_t));
 
 	max_probes = (cmph_uint32)(((log(chd_ph->m)/log(2))/20) * max_probes);
-	
+
 	if(chd_ph->keys_per_bin == 1)
 		chd_ph->occup_table = (cmph_uint8 *) calloc(((chd_ph->n + 31)/32), sizeof(cmph_uint32));
 	else
 		chd_ph->occup_table = (cmph_uint8 *) calloc(chd_ph->n, sizeof(cmph_uint8));
-		
+
 	disp_table = (cmph_uint32 *) calloc(chd_ph->nbuckets, sizeof(cmph_uint32));
-// 	
+// 
 // 	init_genrand(time(0));
-	
+
 	while(1)
 	{
 		iterations --;
@@ -706,12 +706,12 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 		{
 			fprintf(stderr, "Starting mapping step for mph creation of %u keys with %u bins\n", chd_ph->m, chd_ph->n);
 		}
-		
+
 		if(!chd_ph_mapping(mph, buckets, items, &max_bucket_size))
 		{
 			if (mph->verbosity)
 			{
-				fprintf(stderr, "Failure in mapping step\n");		
+				fprintf(stderr, "Failure in mapping step\n");
 			}
 			failure = 1;
 			goto cleanup;
@@ -727,15 +727,15 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 		}
 
         	sorted_lists = chd_ph_ordering(&buckets, &items, chd_ph->nbuckets, chd_ph->m, max_bucket_size);
-		
+
 		if (mph->verbosity)
 		{
 			fprintf(stderr, "Starting searching step\n");
 		}
-		
+
 		searching_success = chd_ph_searching(chd_ph, buckets, items, max_bucket_size, sorted_lists, max_probes, disp_table);
 		if(searching_success) break;
-		
+
 		// reset occup_table
 		if(chd_ph->keys_per_bin > 1)
 			memset(chd_ph->occup_table, 0, chd_ph->n);
@@ -757,19 +757,19 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 	{
 		if(!chd_ph_check_bin_hashing(chd_ph, buckets, items, disp_table,sorted_lists,max_bucket_size))
 		{
-		
+
 			DEBUGP("Error for bin packing generation");
 			failure = 1;
 			goto cleanup;
 		}
 	}
 	#endif
-	
+
 	if (mph->verbosity)
 	{
 		fprintf(stderr, "Starting compressing step\n");
 	}
-	
+
 	if(chd_ph->cs)
 	{
 		free(chd_ph->cs);
@@ -777,7 +777,7 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 	chd_ph->cs = (compressed_seq_t *) calloc(1, sizeof(compressed_seq_t));
 	compressed_seq_init(chd_ph->cs);
 	compressed_seq_generate(chd_ph->cs, disp_table, chd_ph->nbuckets);
-	
+
 	#ifdef CMPH_TIMING
 	ELAPSED_TIME_IN_SECONDS(&construction_time);
 	register double entropy = chd_ph_get_entropy(disp_table, chd_ph->nbuckets, max_probes);
@@ -785,11 +785,11 @@ cmph_t *chd_ph_new(cmph_config_t *mph, double c)
 	#endif
 
 cleanup:
-	chd_ph_bucket_destroy(buckets); 
+	chd_ph_bucket_destroy(buckets);
 	free(items);
 	free(sorted_lists);
 	free(disp_table);
-	if(failure) 
+	if(failure)
 	{
 		if(chd_ph->hl)
 		{
@@ -802,14 +802,14 @@ cleanup:
 	mphf = (cmph_t *)malloc(sizeof(cmph_t));
 	mphf->algo = mph->algo;
 	chd_phf = (chd_ph_data_t *)malloc(sizeof(chd_ph_data_t));
-	
+
 	chd_phf->cs = chd_ph->cs;
 	chd_ph->cs = NULL; //transfer memory ownership
 	chd_phf->hl = chd_ph->hl;
 	chd_ph->hl = NULL; //transfer memory ownership
 	chd_phf->n = chd_ph->n;
 	chd_phf->nbuckets = chd_ph->nbuckets;
-	
+
 	mphf->data = chd_phf;
 	mphf->size = chd_ph->n;
 
@@ -818,12 +818,12 @@ cleanup:
 	{
 		fprintf(stderr, "Successfully generated minimal perfect hash function\n");
 	}
-	
-	#ifdef CMPH_TIMING	
+
+	#ifdef CMPH_TIMING
 	register cmph_uint32 space_usage = chd_ph_packed_size(mphf)*8;
 	construction_time = construction_time - construction_time_begin;
 	fprintf(stdout, "%u\t%.2f\t%u\t%.4f\t%.4f\t%.4f\t%.4f\n", chd_ph->m, load_factor, chd_ph->keys_per_bucket, construction_time, space_usage/(double)chd_ph->m, space_lower_bound, entropy/chd_ph->m);
-	#endif	
+	#endif
 
 	return mphf;
 }
@@ -846,19 +846,19 @@ void chd_ph_load(FILE *fd, cmph_t *mphf)
 	nbytes = fread(buf, (size_t)buflen, (size_t)1, fd);
 	chd_ph->hl = hash_state_load(buf, buflen);
 	free(buf);
-	
+
 	nbytes = fread(&buflen, sizeof(cmph_uint32), (size_t)1, fd);
 	DEBUGP("Compressed sequence structure has %u bytes\n", buflen);
 	buf = (char *)malloc((size_t)buflen);
 	nbytes = fread(buf, (size_t)buflen, (size_t)1, fd);
-	chd_ph->cs = (compressed_seq_t *) calloc(1, sizeof(compressed_seq_t)); 
+	chd_ph->cs = (compressed_seq_t *) calloc(1, sizeof(compressed_seq_t));
 	compressed_seq_load(chd_ph->cs, buf, buflen);
 	free(buf);
-	
+
 	// loading n and nbuckets
 	DEBUGP("Reading n and nbuckets\n");
-	nbytes = fread(&(chd_ph->n), sizeof(cmph_uint32), (size_t)1, fd);	
-	nbytes = fread(&(chd_ph->nbuckets), sizeof(cmph_uint32), (size_t)1, fd);	
+	nbytes = fread(&(chd_ph->n), sizeof(cmph_uint32), (size_t)1, fd);
+	nbytes = fread(&(chd_ph->nbuckets), sizeof(cmph_uint32), (size_t)1, fd);
 }
 
 int chd_ph_dump(cmph_t *mphf, FILE *fd)
@@ -867,7 +867,7 @@ int chd_ph_dump(cmph_t *mphf, FILE *fd)
 	cmph_uint32 buflen;
 	register size_t nbytes;
 	chd_ph_data_t *data = (chd_ph_data_t *)mphf->data;
-	
+
 	__cmph_dump(mphf, fd);
 
 	hash_state_dump(data->hl, &buf, &buflen);
@@ -901,16 +901,16 @@ void chd_ph_destroy(cmph_t *mphf)
 
 cmph_uint32 chd_ph_search(cmph_t *mphf, const char *key, cmph_uint32 keylen)
 {
-	register chd_ph_data_t * chd_ph = mphf->data;
+	register chd_ph_data_t * chd_ph = (chd_ph_data_t *)mphf->data;
 	cmph_uint32 hl[3];
 	register cmph_uint32 disp,position;
 	register cmph_uint32 probe0_num,probe1_num;
 	register cmph_uint32 f,g,h;
-	hash_vector(chd_ph->hl, key, keylen, hl);	
+	hash_vector(chd_ph->hl, key, keylen, hl);
 	g = hl[0] % chd_ph->nbuckets;
 	f = hl[1] % chd_ph->n;
 	h = hl[2] % (chd_ph->n-1) + 1;
-	
+
 	disp = compressed_seq_query(chd_ph->cs, g);
 	probe0_num = disp % chd_ph->n;
 	probe1_num = disp/chd_ph->n;
@@ -921,7 +921,7 @@ cmph_uint32 chd_ph_search(cmph_t *mphf, const char *key, cmph_uint32 keylen)
 void chd_ph_pack(cmph_t *mphf, void *packed_mphf)
 {
 	chd_ph_data_t *data = (chd_ph_data_t *)mphf->data;
-	cmph_uint8 * ptr = packed_mphf;
+	cmph_uint8 * ptr = (cmph_uint8 *)packed_mphf;
 
 	// packing hl type
 	CMPH_HASH hl_type = hash_get_type(data->hl);
@@ -949,40 +949,37 @@ void chd_ph_pack(cmph_t *mphf, void *packed_mphf)
 cmph_uint32 chd_ph_packed_size(cmph_t *mphf)
 {
 	register chd_ph_data_t *data = (chd_ph_data_t *)mphf->data;
-	register CMPH_HASH hl_type = hash_get_type(data->hl); 
+	register CMPH_HASH hl_type = hash_get_type(data->hl);
 	register cmph_uint32 hash_state_pack_size =  hash_state_packed_size(hl_type);
 	register cmph_uint32 cs_pack_size = compressed_seq_packed_size(data->cs);
-	
+
 	return (cmph_uint32)(sizeof(CMPH_ALGO) + hash_state_pack_size + cs_pack_size + 3*sizeof(cmph_uint32));
 
 }
 
 cmph_uint32 chd_ph_search_packed(void *packed_mphf, const char *key, cmph_uint32 keylen)
 {
-	register CMPH_HASH hl_type  = *(cmph_uint32 *)packed_mphf;
+	register CMPH_HASH hl_type  = (CMPH_HASH)*(cmph_uint32 *)packed_mphf;
 	register cmph_uint8 *hl_ptr = (cmph_uint8 *)(packed_mphf) + 4;
-	
+
 	register cmph_uint32 * ptr = (cmph_uint32 *)(hl_ptr + hash_state_packed_size(hl_type));
 	register cmph_uint32 n = *ptr++;
 	register cmph_uint32 nbuckets = *ptr++;
 	cmph_uint32 hl[3];
-		
+
 	register cmph_uint32 disp,position;
 	register cmph_uint32 probe0_num,probe1_num;
 	register cmph_uint32 f,g,h;
-	
+
 	hash_vector_packed(hl_ptr, hl_type, key, keylen, hl);
 
 	g = hl[0] % nbuckets;
 	f = hl[1] % n;
 	h = hl[2] % (n-1) + 1;
-	
+
 	disp = compressed_seq_query_packed(ptr, g);
 	probe0_num = disp % n;
 	probe1_num = disp/n;
 	position = (cmph_uint32)((f + ((cmph_uint64 )h)*probe0_num + probe1_num) % n);
 	return position;
 }
-
-
-
